﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace GenerateOSChecksum
{
    class Startup
    {
        static void Main(string[] args)
        {
            if (args.Length < 2)
            {
                _DisplayUsage();
                return;
            }

            try
            {
                Console.WriteLine("Generating OS checksum...");

                byte checksum = 0;
                for (int i = 1; i < args.Length; i++)
                {
                    var parts = args[i].Split(new char[] { ':' }, StringSplitOptions.RemoveEmptyEntries);
                    var location = Convert.ToInt32(parts[1], 16);
                    var size = Convert.ToInt32(parts[2], 16) - location;
                    var data = new byte[size];

                    var file = new FileStream(parts[0], FileMode.Open);
                    file.Seek(location, SeekOrigin.Begin);
                    file.Read(data, 0, size);
                    file.Close();

                    foreach (var b in data)
                    {
                        checksum += b;
                    }
                }

                checksum = (byte)(0x100 - checksum);
                Console.WriteLine("Checksum is: " + checksum.ToString("X02"));
                Console.WriteLine("Writing to output file...");

                var oparts = args[0].Split(new char[] { ':' });
                var loc = Convert.ToInt32(oparts[1], 16);
                var output = new FileStream(oparts[0], FileMode.Open);
                output.Seek(loc, SeekOrigin.Begin);
                output.Write(new byte[] { checksum }, 0, 1);
                output.Close();

                Console.WriteLine("Done.");
            }
            catch (Exception ex)
            {
                Console.WriteLine("ERROR: " + ex.ToString());
            }
        }

        private static void _DisplayUsage()
        {
            var asm = Assembly.GetExecutingAssembly().GetName();

            Console.WriteLine(string.Format("{0} v{1}", asm.Name, asm.Version.ToString(3)));
            Console.WriteLine("Calculates the OS checksum from a series of files and data ranges,");
            Console.WriteLine("  then embeds it at the specified output file and address.");
            Console.WriteLine();
            Console.WriteLine(string.Format("Usage: {0} output.bin:3FFF file1.bin:0000:4000 file2.bin:0000:4000 ...",
                asm.Name));
            Console.WriteLine();
        }
    }
}
